class GAME {
  private Board b = new Board();
  private Shapes s = new Shapes();
  private Events e = new Events();
  int shapeY; //shape location
  int shapeX;
  boolean gameStarted; //first shape spawned
  boolean spawnShape;
  boolean gameEnded; //to stop draw from drawing frame ontop of black endgame screen after game has actually ended
  boolean landed; //used to trigger newShape (or lose)
  boolean boardAltered; //to determine wheter to draw board or not

  int lineCounter; //lines removed

  void update() { //the game logic
    if(!gameStarted) {
      e.startGame();
    }

    replicaUpdate(); //update our collision Check replica

    if((millis() - gameTime) > 300) { //timer for dropdown
      e.dropDown();
      boardAltered = true;
      gameTime = millis();
    }

    if(!boardAltered)
      keyPressing();

    if(landed) { //attempts to create a new shape (will lose if not possible)
      e.completedLines();
      e.newShape();
      landed = false;
    }
  }
  void display() { //display results of game logic + colors etc... (could move into sepparate tab, like in Events)
    //fill(30,30,30); grid color
    //rect(10,10,(height - 20)/2,(height -20));

    //this is the main grid
    for(int y=0; y<20;y++) {
      for(int x=0; x<10;x++) {  
        if(b.getCell(x+2,y+1) == true) {
          fill(255,255,255);
        }
        if(b.getCell(x+2,y+1) == false) {
          fill(0,0,0);
        }
        rect((10 + (x*cellLength)),(10 + (y*cellLength)),cellLength-1,cellLength-1);
      }
    }
    //fill(30,30,30); grid color
    //rect(20+((height - 20)/2),10+(12*cellLength),(4*cellLength),(4*cellLength));

    //this is the nextShape grid
    for(int y=0; y<4;y++) {
      for(int x=0; x<4;x++) {  
        if(s.nextShape[x][y]) {
          fill(255,255,255);
        }
        else {
          fill(0,0,0);
        }
        rect(20+((height - 20)/2) + (x*cellLength),10+(12*cellLength) + (y*cellLength),cellLength-1,cellLength-1);
      }
    }


    textFont(fancyFont,(width/15));
    fill(255,255,255);
    textAlign(RIGHT);
    text(lineCounter, (width/2)+(10*cellLength), (height/2)-(8*cellLength));

    textFont(fancyFont,(width/15));
    fill(255,255,255);
    textAlign(RIGHT);
    text("Lines:", (width/2)+(10*cellLength), (height/2)-9.2*cellLength);
  }

  void toggleShape() {
    for(int c1=0; c1<4;c1++) {
      for(int c2=0; c2<4;c2++) {
        if(s.SHAPE[c2][c1]) {
          b.toggleCell(c2+shapeX,c1+shapeY);
        }
      }
    }
  }

  /**
   replicaUpdate()
   @param - none= full replica update, 1= partial update
   note: use 1 as parameter as much as possible
   */
  void replicaUpdate() {
    GameLogic.boardReplica = b.cell; //update replicas
    GameLogic.shapeXReplica = shapeX; //update replicas
    GameLogic.shapeYReplica = shapeY; //update replicas
  }

  void replicaUpdate(int x) {
    if(x==1) {
      GameLogic.shapeXReplica = shapeX; //update replicas
      GameLogic.shapeYReplica = shapeY; //update replicas
    }
    else {
      GameLogic.boardReplica = b.cell; //update replicas
      GameLogic.shapeXReplica = shapeX; //update replicas
      GameLogic.shapeYReplica = shapeY; //update replicas
    }
  }

  boolean rotatable =false; //to prevent fast computer from over-interpreting our button mashing
  long rotateTime =0; //this is the "timer" for the boolean above
  boolean droppable =false;
  long dropTime =0;
  boolean movable = false;
  long moveTime =0;

  void keyPressing() { //move this into events later when it works?
    if(keyPressed) {
      switch(key) {
      case 'j': //move left
        if((millis() - moveTime) > 10) { //timer for allowing interaction
          movable = true;
          moveTime = millis();
        }
        if(movable) {
          toggleShape();
          shapeX -= 1;
          replicaUpdate(1);
          if(GameLogic.shapeTestCollide(s.SHAPE)) {
            shapeX += 1;
          }
          toggleShape();
          boardAltered = true;
          movable=false;
          break;
        }

      case 'k': //move right
        if((millis() - moveTime) > 10) { //timer for allowing interaction
          movable = true;
          moveTime = millis();
        }
        if(movable) {
          toggleShape();
          shapeX += 1;
          replicaUpdate(1);
          if(GameLogic.shapeTestCollide(s.SHAPE)) {
            shapeX -= 1;
          }
          toggleShape();
          boardAltered = true;
          movable = false;
          break;
        }

      case 'f': //rotate clock (right)
        if((millis() - rotateTime) > 90) { //timer for allowing interaction
          rotatable = true;
          rotateTime = millis();
        }
        else{break;}
        if(rotatable) {
          toggleShape();
          s.rotateRight();
          s.update();
          if(GameLogic.shapeTestCollide(s.SHAPE)) {
            s.rotateLeft();
            s.update();
          }
          toggleShape();
          boardAltered = true;
          rotatable = false;
          break;
        }        

      case 'd': //rotate c-clock (left)
        if((millis() - rotateTime) > 90) { //timer for allowing interaction
          rotatable = true;
          rotateTime = millis();
        }
        else {break;}
        if(rotatable) {
          toggleShape();
          s.rotateLeft();
          s.update();
          if(GameLogic.shapeTestCollide(s.SHAPE)) {
            s.rotateRight();
            s.update();
          }
          toggleShape();
          boardAltered = true;
          rotatable = false;
          break;
        }

      case ' ': //drop shape
        if((millis() - dropTime) > 90) { //timer for allowing interaction
          droppable = true;
          dropTime = millis();
        }
        if(droppable) {
          while(!landed)
            e.dropDown();
          droppable = false;  
          break;
        }
      }
    }
  }


  GAME() {
    gameStarted = false;
  }
}

